from lxml import etree
from .osm_features import OSM_FEATURES

class OSMStats(object):

    def __init__(self, osm_file):
        self.osm_file = osm_file
        self.xml_data = etree.parse(self.osm_file)
        self.bounds = self.xml_data.xpath("//bounds")[0].attrib
        self.contributors = {}
        self.details = {}

    def get_contributors(self):
        '''Returns all contributors of this map,
           with their contribution count.

           :return: contributors with contribution count
           :rtype: dict
        '''
        if not self.contributors:
            for user in self.xml_data.xpath("//@user"):
                if user in self.contributors.keys():
                    self.contributors[user] += 1
                else:
                    self.contributors[user] = 1
        return self.contributors

    def get_count(self, feature_name, feature_type=None):
        '''Counts the total number of feature (or) a
           specific type of the feature supplied.

           :param feature_name: name of the OSM feature
           :param feature_type: name of the OSM sub-feature

           :example: get_count('amenity', 'pharmacy')
           :return: total number of feature (or) sub-feature
           :rtype: int
        '''
        if feature_type:
            s_query = "//tag[@k='{}' and @v='{}']".format(feature_name, feature_type)
        else:
            s_query = "//tag[@k='{}']".format(feature_name)
        return len(self.xml_data.xpath(s_query))

    def get_feature(self, feature_name):
        '''Returns sub-features of the feature with
           individual count for each sub-features.

           This method also incrementally builds the
           self.details dictionary with new feature names
           if not already present.

           :param feature_name: name of the OSM feature

           :example: get_feature('shops')
           :return: sub-features and it's count for the feature
           :rtype: dict
        '''
        if feature_name not in OSM_FEATURES:
            return "Unknown feature %s" %feature_name

        if feature_name not in self.details:
            self.details[feature_name] = {}
            for tag in self.xml_data.xpath("//tag[@k='%s']" %feature_name):
                if tag.attrib['v'] in self.details[feature_name]:
                    self.details[feature_name][tag.attrib['v']] += 1
                else:
                    self.details[feature_name][tag.attrib['v']] = 1
        return self.details[feature_name]



'''
These features and their sub-features are the reflection
of Mapping Features in OSM as mentioned in their wiki.

Refer: http://wiki.openstreetmap.org/wiki/Map_Features
'''

OSM_FEATURES = {
        "aerialway": ["cable_car", "gondola",
                      "chair_lift", "mixed_lift",
                      "drag_lift", "t-bar", "j-bar",
                      "platter", "rope_tow", "magic_carpet",
                      "zip_line", "pylon", "station", "canopy",
                      "goods"],

        "aeroway": ["aerodrome", "apron", "gate", "hangar",
                    "helipad", "heliport", "navigationaid", "runway",
                    "spaceport", "taxilane", "taxiway", "terminal",
                    "windsock"],

        "amenity": ["bar", "bbq", "biergarten", "cafe", "drinking_water",
                     "fast_food", "food_court", "ice_cream", "pub",
                     "restaurant", "college", "kindergarten", "library",
                     "archive", "public_bookcase", "school", "music_school",
                     "driving_school", "language_school", "university",
                     "bicycle_parking", "bicycle_repair_station",
                     "bicycle_rental", "boat_rental", "boat_sharing",
                     "bus_station", "car_rental", "car_sharing",
                     "car_wash", "charging_station", "ferry_terminal",
                     "fuel", "grit_bin", "motorcycle_parking", "parking",
                     "parking_entrance", "parking_space", "taxi", "atm",
                     "bank", "bureau_de_change", "baby_box", "clinic", 
                     "dentist", "doctors", "hospital", "nursing_home",
                     "pharmacy", "social_facility", "veterinary",
                     "blood_donation", "arts_center", "brothel",
                     "casino", "cinema", "community_center", "fountain",
                     "gambling", "nightclub", "planetarium", "social_center",
                     "stripclub", "studio", "swingerclub", "theatre",
                     "animal_boarding", "animal_shelter", "baking_oven",
                     "bench", "clock", "courthouse", "coworking_space",
                     "crematorium", "crypt", "dive_center", "dojo",
                     "embassy", "fire_station", "game_feeding", "grave_yard",
                     "hunting_stand", "internet_cafe", "kneipp_water_cure",
                     "marketplace", "photo_booth", "place_of_worship",
                     "police", "post_box", "prison", "public_bath",
                     "ranger_station", "recycling", "rescue_station",
                     "sanitary_dump_station", "shelter", "shower", "table",
                     "telephone", "toilets", "townhall", "vending_machine",
                     "waste_basket", "waste_disposal",
                     "waste_transfer_station", "watering_place", "water_point"],
        "barrier": [],
        "boundary": [],
        "building": [],
        "craft": [],
        "emergency": [],
        "geological": [],
        "highway": [],
        "historic": [],
        "landuse": [],
        "leisure": [],
        "man_made": [],
        "military": [],
        "natural": [],
        "office": [],
        "place": [],
        "power": [],
        "public_transport": [],
        "railway": [],
        "route": [],
        "shop": [],
        "sport": [],
        "tourism": [],
        "waterway": []
            }
